import sys
import os
from unittest.mock import (
    patch, Mock, MagicMock
)

import kiwi

from ..test_helper import argv_kiwi_tests

from kiwi.defaults import Defaults
from kiwi.tasks.system_update import SystemUpdateTask


class TestSystemUpdateTask:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        sys.argv = [
            sys.argv[0], '--profile', 'vmxFlavour', 'system', 'update',
            '--root', '../data/root-dir'
        ]
        self.abs_target_dir = os.path.abspath('some-target')

        kiwi.tasks.system_update.Privileges = Mock()
        kiwi.tasks.system_update.Path = Mock()

        kiwi.tasks.system_update.Help = Mock(
            return_value=Mock()
        )
        self.task = SystemUpdateTask()

    def setup_method(self, cls):
        self.setup()

    def teardown(self):
        sys.argv = argv_kiwi_tests

    def teardown_method(self, cls):
        self.teardown()

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['update'] = False
        self.task.command_args['--add-package'] = []
        self.task.command_args['--delete-package'] = []
        self.task.command_args['--root'] = '../data/root-dir'

    @patch('kiwi.tasks.system_update.SystemPrepare')
    def test_process_system_update(self, mock_SystemPrepare):
        manager = MagicMock()
        system_prepare = Mock()
        system_prepare.setup_repositories = Mock(
            return_value=manager
        )
        mock_SystemPrepare.return_value.__enter__.return_value = system_prepare
        self._init_command_args()
        self.task.command_args['update'] = True
        self.task.process()
        system_prepare.setup_repositories.assert_called_once_with(
            target_arch=None
        )
        system_prepare.update_system.assert_called_once_with(
            manager.__enter__.return_value
        )

    @patch('kiwi.tasks.system_update.SystemPrepare')
    def test_process_system_update_add_package(self, mock_SystemPrepare):
        manager = MagicMock()
        system_prepare = Mock()
        system_prepare.setup_repositories = Mock(
            return_value=manager
        )
        mock_SystemPrepare.return_value.__enter__.return_value = system_prepare
        self._init_command_args()
        self.task.command_args['update'] = True
        self.task.command_args['--add-package'] = ['vim']
        self.task.process()
        system_prepare.setup_repositories.assert_called_once_with(
            target_arch=None
        )
        system_prepare.install_packages.assert_called_once_with(
            manager.__enter__.return_value, ['vim']
        )

    @patch('kiwi.tasks.system_update.SystemPrepare')
    def test_process_system_update_delete_package(self, mock_SystemPrepare):
        manager = MagicMock()
        system_prepare = Mock()
        system_prepare.setup_repositories = Mock(
            return_value=manager
        )
        mock_SystemPrepare.return_value.__enter__.return_value = system_prepare
        self._init_command_args()
        self.task.command_args['update'] = True
        self.task.command_args['--delete-package'] = ['vim']
        self.task.process()
        system_prepare.setup_repositories.assert_called_once_with(
            target_arch=None
        )
        system_prepare.delete_packages.assert_called_once_with(
            manager.__enter__.return_value, ['vim']
        )

    def test_process_system_update_help(self):
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['update'] = True
        self.task.process()
        self.task.manual.show.assert_called_once_with(
            'kiwi::system::update'
        )



import sys
from unittest.mock import (
    Mock, MagicMock
)
import os

import kiwi

from ..test_helper import argv_kiwi_tests

from kiwi.tasks.system_create import SystemCreateTask


class TestSystemCreateTask:
    def setup(self):
        sys.argv = [
            sys.argv[0], '--profile', 'vmxFlavour', 'system', 'create',
            '--root', '../data/root-dir', '--target-dir', 'some-target'
        ]

        self.abs_root_dir = os.path.abspath('../data/root-dir')
        self.abs_target_dir = os.path.abspath('some-target')

        kiwi.tasks.system_create.Path = Mock()
        kiwi.tasks.system_create.Privileges = Mock()

        kiwi.tasks.system_create.Help = Mock(
            return_value=Mock()
        )

        self.runtime_checker = Mock()
        kiwi.tasks.base.RuntimeChecker = Mock(
            return_value=self.runtime_checker
        )

        self.runtime_config = Mock()
        self.runtime_config.get_disabled_runtime_checks.return_value = []
        kiwi.tasks.base.RuntimeConfig = Mock(
            return_value=self.runtime_config
        )

        self.setup = Mock()
        kiwi.tasks.system_create.SystemSetup = Mock(
            return_value=self.setup
        )

        self.result = Mock()
        self.builder = MagicMock()
        self.builder.create = Mock(
            return_value=self.result
        )
        kiwi.tasks.system_create.ImageBuilder.new = Mock(
            return_value=self.builder
        )

        self.task = SystemCreateTask()

    def setup_method(self, cls):
        self.setup()

    def teardown(self):
        sys.argv = argv_kiwi_tests

    def teardown_method(self, cls):
        self.teardown()

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['create'] = False
        self.task.command_args['--root'] = '../data/root-dir'
        self.task.command_args['--target-dir'] = 'some-target'
        self.task.command_args['--signing-key'] = ['some-key-file']

    def test_process_system_create(self):
        self._init_command_args()
        self.task.command_args['create'] = True
        self.task.process()
        self.runtime_checker.\
            check_target_directory_not_in_shared_cache.\
            assert_called_once_with(self.abs_root_dir)
        self.runtime_checker.\
            check_dracut_module_versions_compatible_to_kiwi.\
            assert_called_once_with(self.abs_root_dir)
        self.setup.call_image_script.assert_called_once_with()
        self.builder.create.assert_called_once_with()
        self.result.print_results.assert_called_once_with()
        self.result.dump.assert_called_once_with(
            os.sep.join([self.abs_target_dir, 'kiwi.result'])
        )

    def test_process_system_create_help(self):
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['create'] = True
        self.task.process()
        self.task.manual.show.assert_called_once_with(
            'kiwi::system::create'
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import logging
from typing import Dict

# project
from kiwi.defaults import Defaults
from kiwi.archive.tar import ArchiveTar
from kiwi.archive.cpio import ArchiveCpio
from kiwi.system.setup import SystemSetup
from kiwi.system.result import Result
from kiwi.runtime_config import RuntimeConfig
from kiwi.xml_state import XMLState

from kiwi.exceptions import (
    KiwiArchiveSetupError
)

log = logging.getLogger('kiwi')


class ArchiveBuilder:
    """
    **Root archive image builder**

    :param object xml_state: Instance of :class:`XMLState`
    :param str target_dir: target directory path name
    :param str root_dir: root directory path name
    :param dict custom_args: Custom processing arguments defined as hash keys:
        * xz_options: string of XZ compression parameters
    """
    def __init__(
        self, xml_state: XMLState, target_dir: str,
        root_dir: str, custom_args: Dict = None
    ):
        self.root_dir = root_dir
        self.target_dir = target_dir
        self.xml_state = xml_state
        self.requested_archive_type = xml_state.get_build_type_name()
        self.bundle_format = xml_state.get_build_type_bundle_format()
        self.result = Result(xml_state)
        self.system_setup = SystemSetup(
            xml_state=xml_state, root_dir=self.root_dir
        )
        self.filename = ''
        self.xz_options = custom_args['xz_options'] if custom_args \
            and 'xz_options' in custom_args else None

        self.runtime_config = RuntimeConfig()

    def create(self) -> Result:
        """
        Create a root archive tarball

        Build a simple XZ compressed root tarball from the image root tree

        Image types which triggers this builder are:

        * image="tbz"
        * image="cpio"

        :return: result

        :rtype: instance of :class:`Result`
        """
        supported_archives = Defaults.get_archive_image_types()
        if self.requested_archive_type not in supported_archives:
            raise KiwiArchiveSetupError(
                'Unknown archive type: %s' % self.requested_archive_type
            )

        if self.requested_archive_type == 'tbz':
            log.info('Creating XZ compressed tar archive')
            self.filename = self._target_file_for('tar.xz')
            ArchiveTar(
                self._target_file_for('tar')
            ).create_xz_compressed(
                self.root_dir, xz_options=self.xz_options,
                exclude=Defaults.
                get_exclude_list_for_root_data_sync() + Defaults.
                get_exclude_list_from_custom_exclude_files(self.root_dir)
            )
        if self.requested_archive_type == 'cpio':
            log.info('Creating CPIO archive')
            self.filename = self._target_file_for('cpio')
            ArchiveCpio(self.filename).create(
                self.root_dir,
                exclude=Defaults.
                get_exclude_list_for_root_data_sync() + Defaults.
                get_exclude_list_from_custom_exclude_files(self.root_dir)
            )
        if self.filename:
            Result.verify_image_size(
                self.runtime_config.get_max_size_constraint(),
                self.filename
            )
            if self.bundle_format:
                self.result.add_bundle_format(self.bundle_format)
            self.result.add(
                key='root_archive',
                filename=self.filename,
                use_for_bundle=True,
                compress=False,
                shasum=True
            )
            self.result.add(
                key='image_packages',
                filename=self.system_setup.export_package_list(
                    self.target_dir
                ),
                use_for_bundle=True,
                compress=False,
                shasum=False
            )
            self.result.add(
                key='image_changes',
                filename=self.system_setup.export_package_changes(
                    self.target_dir
                ),
                use_for_bundle=True,
                compress=True,
                shasum=False
            )
            self.result.add(
                key='image_verified',
                filename=self.system_setup.export_package_verification(
                    self.target_dir
                ),
                use_for_bundle=True,
                compress=False,
                shasum=False
            )
        return self.result

    def _target_file_for(self, suffix: str) -> str:
        return ''.join(
            [
                self.target_dir, '/',
                self.xml_state.xml_data.get_name(),
                '.' + Defaults.get_platform_name(),
                '-' + self.xml_state.get_image_version(),
                '.', suffix
            ]
        )



# Copyright (c) 2020 SUSE Software Solutions Germany GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import logging
from typing import Dict

# project
from kiwi.utils.temporary import Temporary
from kiwi.archive.tar import ArchiveTar
from kiwi.defaults import Defaults
from kiwi.utils.compress import Compress
from kiwi.runtime_config import RuntimeConfig
from kiwi.command import Command
from kiwi.container.base import ContainerImageBase
from kiwi.exceptions import KiwiContainerSetupError

log = logging.getLogger('kiwi')


class ContainerImageAppx(ContainerImageBase):
    """
    Create Appx container from a root directory for
    WSL(Windows Subsystem Linux)

    :param string root_dir: root directory path name
    :param dict custom_args:

    Custom processing arguments defined as hash keys:

    Example

    .. code:: python

        {
            'metadata_path': 'directory'
        }
    """
    def __init__(self, root_dir: str, custom_args: Dict[str, str] = {}):
        self.root_dir = root_dir
        self.wsl_config = custom_args or {}
        self.runtime_config = RuntimeConfig()

        self.meta_data_path = format(self.wsl_config.get('metadata_path') or '')

        if not self.meta_data_path:
            raise KiwiContainerSetupError(
                'No metadata path specified to build appx container'
            )

        if not os.path.exists(self.meta_data_path):
            raise KiwiContainerSetupError(
                'Specified metadata path {0} does not exist'.format(
                    self.meta_data_path
                )
            )

    def create(
        self, filename: str, base_image: str = '',
        ensure_empty_tmpdirs: bool = False, compress_archive: bool = False
    ) -> str:
        """
        Create WSL/Appx archive

        :param string filename: archive file name
        :param string base_image: not-supported
        :param bool ensure_empty_tmpdirs: not-supported
        :param bool compress_archive: compress container archive
        """
        exclude_list = Defaults.\
            get_exclude_list_for_root_data_sync() + Defaults.\
            get_exclude_list_from_custom_exclude_files(self.root_dir)
        exclude_list.append('boot')
        exclude_list.append('dev')
        exclude_list.append('sys')
        exclude_list.append('proc')

        # The C code of WSL-DistroLauncher harcodes the name for the
        # root tarball to be install.tar.gz. Thus we have to use this
        # name for the root tarball
        archive_file_name = os.sep.join(
            [self.meta_data_path, 'install.tar']
        )
        archive = ArchiveTar(
            archive_file_name
        )
        archive_file_name = archive.create(
            self.root_dir, exclude=exclude_list
        )
        compressor = Compress(archive_file_name)
        archive_file_name = compressor.gzip()

        filemap_file = Temporary().new_file()
        with open(filemap_file.name, 'w') as filemap:
            filemap.write('[Files]{0}'.format(os.linesep))
            for topdir, dirs, files in sorted(os.walk(self.meta_data_path)):
                for entry in sorted(dirs + files):
                    if entry in files:
                        mapfile = os.sep.join([topdir, entry])
                        mapfile_relative = os.path.relpath(mapfile, start=self.meta_data_path)
                        log.info(
                            'Adding {0} to Appx filemap as relative path {1}'.format(mapfile, mapfile_relative)
                        )
                        filemap.write(
                            '"{0}" "{1}"{2}'.format(
                                mapfile, mapfile_relative, os.linesep
                            )
                        )

        Command.run(
            ['appx', '-o', filename, '-f', filemap_file.name]
        )
        if compress_archive:
            compress = Compress(filename)
            filename = compress.xz(self.runtime_config.get_xz_options())

        return filename



from unittest.mock import (
    patch, Mock
)
from pytest import raises

from kiwi.builder import ImageBuilder

from kiwi.exceptions import KiwiRequestedTypeError


class TestImageBuilder:
    @patch('kiwi.builder.filesystem.FileSystemBuilder')
    def test_filesystem_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='ext4'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    @patch('kiwi.builder.disk.DiskBuilder')
    def test_disk_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='oem'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    @patch('kiwi.builder.live.LiveImageBuilder')
    def test_live_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='iso'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    @patch('kiwi.builder.kis.KisBuilder')
    def test_kis_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='kis'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )
        mock_builder.reset_mock()
        xml_state.get_build_type_name = Mock(
            return_value='pxe'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    @patch('kiwi.builder.archive.ArchiveBuilder')
    def test_archive_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='tbz'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    @patch('kiwi.builder.container.ContainerBuilder')
    def test_container_builder(self, mock_builder):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='docker'
        )
        ImageBuilder.new(xml_state, 'target_dir', 'root_dir')
        mock_builder.assert_called_once_with(
            xml_state, 'target_dir', 'root_dir', None
        )

    def test_unsupported_build_type(self):
        xml_state = Mock()
        xml_state.get_build_type_name = Mock(
            return_value='bogus'
        )
        with raises(KiwiRequestedTypeError):
            ImageBuilder.new(xml_state, 'target_dir', 'root_dir')



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import logging
from typing import (
    Dict, List, Optional, Union
)

# project
from kiwi.iso_tools.base import IsoToolsBase
from kiwi.path import Path
from kiwi.command import Command
from kiwi.exceptions import KiwiIsoToolError
from kiwi.defaults import Defaults

log = logging.getLogger('kiwi')


class IsoToolsXorrIso(IsoToolsBase):
    """
    **xorriso wrapper class**

    Implementation of Parameter API for iso creation tools using
    the libburnia project. Addressed here is the tool xorriso
    """
    def has_iso_hybrid_capability(self) -> bool:
        """
        Indicate if the iso tool has the capability to embed a
        partition table into the iso such that it can be
        used as both; an iso and a disk

        :return: True or False

        :rtype: bool
        """
        return True

    def get_tool_name(self) -> str:
        """
        Lookup xorriso in search path

        :raises KiwiIsoToolError: if xorriso tool is not found
        :return: xorriso tool path

        :rtype: str
        """
        xorriso = Path.which('xorriso')
        if xorriso:
            return xorriso

        raise KiwiIsoToolError('xorriso tool not found')

    def init_iso_creation_parameters(
        self, custom_args: Optional[Dict[str, Union[str, bool]]] = None
    ) -> None:
        """
        Create a set of standard parameters

        :param list custom_args: custom ISO meta data
        """
        legacy_bios_mode = True
        if custom_args:
            application_id = \
                custom_args.get('application_id') or custom_args.get('mbr_id')
            if application_id:
                self.iso_parameters += [
                    '-application_id', format(application_id)
                ]
            if 'publisher' in custom_args:
                self.iso_parameters += [
                    '-publisher', format(custom_args['publisher'])
                ]
            if 'preparer' in custom_args:
                self.iso_parameters += [
                    '-preparer_id', format(custom_args['preparer'])
                ]
            if 'volume_id' in custom_args:
                self.iso_parameters += [
                    '-volid', format(custom_args['volume_id'])
                ]
            if 'legacy_bios_mode' in custom_args:
                legacy_bios_mode = bool(custom_args['legacy_bios_mode'])
        catalog_file = self.boot_path + '/boot.catalog'
        self.iso_parameters += [
            '-joliet', 'on', '-padding', '0'
        ]
        if Defaults.is_ppc64_arch(self.arch):
            self.iso_parameters += [
                '-compliance', 'untranslated_names'
            ]

        if Defaults.is_x86_arch(self.arch) and legacy_bios_mode:
            mbr_file = os.sep.join(
                [
                    self.source_dir, self.boot_path, 'loader',
                    Defaults.get_iso_grub_mbr()
                ]
            )
            loader_file = os.sep.join(
                [
                    self.boot_path, 'loader',
                    Defaults.get_iso_grub_loader()
                ]
            )
            self.iso_loaders += [
                '-boot_image', 'grub', 'bin_path={0}'.format(loader_file)
            ]
            if os.path.exists(mbr_file):
                self.iso_loaders += [
                    '-boot_image', 'grub', 'grub2_mbr={0}'.format(mbr_file)
                ]
            else:
                log.warning(f'No hybrid MBR file found: {mbr_file}: skipped')
            self.iso_loaders += [
                '-boot_image', 'grub', 'grub2_boot_info=on'
            ]

        if Defaults.is_ppc64_arch(self.arch):
            self.iso_loaders += [
                '-boot_image', 'any', 'chrp_boot_part=on'
            ]
        else:
            self.iso_loaders += [
                '-boot_image', 'any', 'partition_offset=16',
                '-boot_image', 'any', 'cat_path={0}'.format(catalog_file),
                '-boot_image', 'any', 'cat_hidden=on',
                '-boot_image', 'any', 'boot_info_table=on',
                '-boot_image', 'any', 'platform_id=0x00',
                '-boot_image', 'any', 'emul_type=no_emulation',
                '-boot_image', 'any', 'load_size=2048'
            ]

    def add_efi_loader_parameters(self, loader_file: str) -> None:
        """
        Add ISO creation parameters to embed the EFI loader

        In order to boot the ISO from EFI, the EFI binary is added as
        alternative loader to the ISO creation parameter list. The
        EFI binary must be included into a fat filesystem in order
        to become recognized by the firmware. For details about this
        file refer to _create_embedded_fat_efi_image() from
        bootloader/config/grub2.py
        """
        self.iso_loaders += [
            '-append_partition', '2', '0xef', loader_file,
            '-boot_image', 'any', 'next',
            '-boot_image', 'any',
            'efi_path=--interval:appended_partition_2:all::',
            '-boot_image', 'any', 'platform_id=0xef',
            '-boot_image', 'any', 'emul_type=no_emulation'
        ]

    def create_iso(
        self, filename: str, hidden_files: List[str] = None
    ) -> None:
        """
        Creates the iso file with the given filename using xorriso

        :param str filename: output filename
        :param list hidden_files: list of hidden files
        """
        hidden_files_parameters = []
        if hidden_files:
            for hidden_file in hidden_files:
                hidden_files_parameters += [
                    '--', '-find', hidden_file, '-exec', 'hide', 'on'
                ]
        Path.wipe(filename)
        Command.run(
            [
                self.get_tool_name()
            ] + self.iso_parameters + [
                '-outdev', filename, '-map', self.source_dir, '/',
                '-chmod', '0755', '/', '--'
            ] + self.iso_loaders + hidden_files_parameters
        )



from unittest.mock import patch, Mock

from kiwi.defaults import Defaults
from kiwi.iso_tools.iso import Iso


class TestIso:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        self.iso = Iso('source-dir')

    def setup_method(self, cls):
        self.setup()

    @patch('kiwi.iso_tools.iso.Command.run')
    @patch('kiwi.iso_tools.iso.RuntimeConfig')
    def test_set_media_tag_checkmedia(self, mock_RuntimeConfig, mock_command):
        runtime_config = Mock()
        runtime_config.get_iso_media_tag_tool.return_value = 'checkmedia'
        mock_RuntimeConfig.return_value = runtime_config
        Iso.set_media_tag('foo')
        mock_command.assert_called_once_with(
            ['tagmedia', '--digest', 'sha256', '--check', '--pad', '0', 'foo']
        )

    @patch('kiwi.iso_tools.iso.Command.run')
    @patch('kiwi.iso_tools.iso.RuntimeConfig')
    def test_set_media_tag_isomd5sum(self, mock_RuntimeConfig, mock_command):
        runtime_config = Mock()
        runtime_config.get_iso_media_tag_tool.return_value = 'isomd5sum'
        mock_RuntimeConfig.return_value = runtime_config
        Iso.set_media_tag('foo')
        mock_command.assert_called_once_with(
            ['implantisomd5', '--force', 'foo']
        )



# Copyright (c) 2020 SUSE LLC.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import importlib
from abc import (
    ABCMeta,
    abstractmethod
)
import logging

# project
from kiwi.exceptions import KiwiAnyMarkupPluginError

log = logging.getLogger('kiwi')


class Markup(metaclass=ABCMeta):
    """
    **Markup factory**

    :param str description: path to description file
    """
    @abstractmethod
    def __init__(self) -> None:
        return None  # pragma: no cover

    @staticmethod
    def new(description: str, name: str='any'):  # noqa: E252
        try:
            markup = Markup._load_markup_by_name(name, description)
            log.info('Support for multiple markup descriptions available')
        except KiwiAnyMarkupPluginError:
            markup = Markup._load_markup_by_name('xml', description)
            log.info('Support for XML markup available')
        return markup

    @staticmethod
    def _load_markup_by_name(name, description):
        name_map = {
            'any': 'Any',
            'xml': 'XML'
        }
        markup = importlib.import_module('kiwi.markup.{0}'.format(name))
        return markup.__dict__['Markup{0}'.format(name_map[name])](
            description
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from .tasks.base import CliTask


class App:
    """
    **Implements creation of task instances**

    Each task class implements a process method which is called
    when constructing an instance of App
    """
    def __init__(self):
        app = CliTask(should_perform_task_setup=False)
        action = app.cli.get_command()
        service = app.cli.get_servicename()
        task_class_name = service.title() + action.title() + 'Task'
        task_class = app.task.__dict__[task_class_name]
        task_class().process()



from unittest.mock import (
    patch, call
)

import unittest.mock as mock

from kiwi.filesystem.ext2 import FileSystemExt2


class TestFileSystemExt2:
    @patch('os.path.exists')
    def setup(self, mock_exists):
        mock_exists.return_value = True
        provider = mock.Mock()
        provider.get_device = mock.Mock(
            return_value='/dev/foo'
        )
        self.ext2 = FileSystemExt2(provider, 'root_dir')
        self.ext2.setup_mountpoint = mock.Mock(
            return_value='some-mount-point'
        )

    @patch('os.path.exists')
    def setup_method(self, cls, mock_exists):
        self.setup()

    @patch('kiwi.filesystem.ext2.Command.run')
    def test_create_on_device(self, mock_command):
        self.ext2.create_on_device('label', 100, uuid='uuid')
        call = mock_command.call_args_list[0]
        assert mock_command.call_args_list[0] == \
            call(['mkfs.ext2', '-L', 'label', '-U', 'uuid', '/dev/foo', '100'])

    @patch('kiwi.filesystem.ext2.Command.run')
    def test_set_uuid(self, mock_command):
        self.ext2.set_uuid()
        assert mock_command.call_args_list == [
            call(['e2fsck', '-y', '-f', '/dev/foo'], raise_on_error=False),
            call(['tune2fs', '-f', '-U', 'random', '/dev/foo'])
        ]
